% Himmelblau's function
% 
% f(x1,x2) = (x1.^2 + x2 - 11).^2 + (x1 + x2.^2 - 7).^2
% 
% 4 local minima: (approximate values)
% xa = [ 3.0      ;  2.0     ]  with  f(xa) = 0.0
% xb = [ 2.805118 ;  3.131312]  with  f(xb) = 0.0
% xc = [-3.779310 ; -3.283186]  with  f(xc) = 0.0 
% xd = [ 3.584428 ; -1.848126]  with  f(xd) = 0.0
% 
% 1 local maximum:
% xm = [-0.270845 ; -0.923039]  with  f(xm) = 181.617
% 
% 

% prepare figure
figure(338); clf; hold('on');

% draw Himmelblau's function
fv = @(x1, x2) (x1.^2 + x2 - 11).^2 + (x1 + x2.^2 - 7).^2;
grid = -6:0.1:+6;
[x1, x2] = meshgrid(grid, grid);
z = fv(x1,x2); 
z(z>560) = NaN;    % for better coloring
h = surf(x1, x2, z);
h.EdgeAlpha = 0.1;
h.FaceAlpha = 0.5;
colormap('jet');   % coloring
zlim([0, 500]);    % clipping


% insert the extrema
xa = [ 3.0      ;  2.0     ];
xb = [-2.805118 ;  3.131312];
xc = [-3.779310 ; -3.283186];
xd = [ 3.584428 ; -1.848126];
xm = [-0.270845 ; -0.923039];  % maximum
markersize = 50;
plot3(xa(1),xa(2),fv(xa(1),xa(2)), 'b.', 'MarkerSize', markersize);
plot3(xb(1),xb(2),fv(xb(1),xb(2)), 'b.', 'MarkerSize', markersize);
plot3(xc(1),xc(2),fv(xc(1),xc(2)), 'b.', 'MarkerSize', markersize);
plot3(xd(1),xd(2),fv(xd(1),xd(2)), 'b.', 'MarkerSize', markersize);
plot3(xm(1),xm(2),fv(xm(1),xm(2)), 'r.', 'MarkerSize', markersize);

